@@ -235,7 +235,7 @@ static void rugged_repo_new_with_backend(git_repository **repo, VALUE rb_path, V
 
 	if (error == GIT_ENOTFOUND) {
 		giterr_clear();
-		error = git_reference_symbolic_create(&head, *repo, "HEAD", "refs/heads/master", 0, NULL, NULL);
+		error = git_reference_symbolic_create(&head, *repo, "HEAD", "refs/heads/master", 0, NULL);
 	}
 
 	if (!error) {
@@ -410,7 +410,6 @@ static VALUE rb_git_repo_init_at(int argc, VALUE *argv, VALUE klass)
 
 static void parse_clone_options(git_clone_options *ret, VALUE rb_options, struct rugged_remote_cb_payload *remote_payload)
 {
-	git_remote_callbacks remote_callbacks = GIT_REMOTE_CALLBACKS_INIT;
 	VALUE val;
 
 	if (NIL_P(rb_options))
@@ -426,9 +425,7 @@ static void parse_clone_options(git_clone_options *ret, VALUE rb_options, struct
 		ret->checkout_branch = StringValueCStr(val);
 	}
 
-	rugged_remote_init_callbacks_and_payload_from_options(rb_options, &remote_callbacks, remote_payload);
-
-	ret->remote_callbacks = remote_callbacks;
+	rugged_remote_init_callbacks_and_payload_from_options(rb_options, &ret->fetch_opts.callbacks, remote_payload);
 }
 
 /*
@@ -593,6 +590,75 @@ static VALUE rb_git_repo_get_config(VALUE self)
 	RB_GIT_REPO_OWNED_GET(rb_cRuggedConfig, config);
 }
 
+/*
+ *  call-seq:
+ *    repo.ident = ident
+ *
+ *  Set the identity to be used for writing reflogs.
+ *
+ *  +ident+ can be either +nil+ or a Hash containing +name+ and/or +email+ entries.
+ */
+static VALUE rb_git_repo_set_ident(VALUE self, VALUE rb_ident) {
+	VALUE rb_val;
+
+	git_repository *repo;
+	const char *name = NULL, *email = NULL;
+
+	Data_Get_Struct(self, git_repository, repo);
+
+	if (!NIL_P(rb_ident)) {
+		Check_Type(rb_ident, T_HASH);
+
+		if (!NIL_P(rb_val = rb_hash_aref(rb_ident, CSTR2SYM("name")))) {
+			Check_Type(rb_val, T_STRING);
+			name = StringValueCStr(rb_val);
+		}
+
+		if (!NIL_P(rb_val = rb_hash_aref(rb_ident, CSTR2SYM("email")))) {
+			Check_Type(rb_val, T_STRING);
+			email = StringValueCStr(rb_val);
+		}
+	}
+
+	rugged_exception_check(
+		git_repository_set_ident(repo, name, email)
+	);
+
+	return Qnil;
+}
+
+/*
+ *  call-seq:
+ *    repo.ident -> ident
+ *
+ *  Return a Hash containing the identity that is used to write reflogs.
+ *
+ *  +ident+ is a Hash containing +name+ and/or +email+ entries, or `nil`.
+ */
+static VALUE rb_git_repo_get_ident(VALUE self)
+{
+	VALUE rb_ident = rb_hash_new();
+
+	git_repository *repo;
+	const char *name = NULL, *email = NULL;
+
+	Data_Get_Struct(self, git_repository, repo);
+
+	rugged_exception_check(
+		git_repository_ident(&name, &email, repo)
+	);
+
+	if (name) {
+		rb_hash_aset(rb_ident, CSTR2SYM("name"), rb_str_new_utf8(name));
+	}
+
+	if (email) {
+		rb_hash_aset(rb_ident, CSTR2SYM("email"), rb_str_new_utf8(email));
+	}
+
+	return rb_ident;
+}
+
 /*
  *  call-seq:
  *    repo.merge_base(oid1, oid2, ...)
@@ -1194,7 +1260,7 @@ static VALUE rb_git_repo_set_head(VALUE self, VALUE rb_head)
 	Data_Get_Struct(self, git_repository, repo);
 
 	Check_Type(rb_head, T_STRING);
-	error = git_repository_set_head(repo, StringValueCStr(rb_head), NULL, NULL);
+	error = git_repository_set_head(repo, StringValueCStr(rb_head));
 	rugged_exception_check(error);
 
 	return Qnil;
@@ -1517,7 +1583,7 @@ static int parse_reset_type(VALUE rb_reset_type)
 
 /*
  *  call-seq:
- *    repo.reset(target, reset_type, options = {}) -> nil
+ *    repo.reset(target, reset_type) -> nil
  *
  *  Sets the current head to the specified commit oid and optionally
  *  resets the index and working tree to match.
@@ -1534,51 +1600,25 @@ static int parse_reset_type(VALUE rb_reset_type)
  *    replaced with the content of the index. (Untracked and ignored files
  *    will be left alone)
  *
- *  The following options can be passed in the +options+ Hash:
- *
- *  :message ::
- *    A single line log message to be appended to the reflog.
- *
- *  :signature ::
- *    The signature to be used for populating the reflog entry.
- *
  *  Examples:
  *
  *    repo.reset('origin/master', :hard) #=> nil
  */
-static VALUE rb_git_repo_reset(int argc, VALUE *argv, VALUE self)
+static VALUE rb_git_repo_reset(VALUE self, VALUE rb_target, VALUE rb_reset_type)
 {
-	VALUE rb_target, rb_reset_type, rb_options;
 	git_repository *repo;
 	int reset_type;
 	git_object *target = NULL;
-	char *log_message = NULL;
-	git_signature *signature = NULL;
 	int error;
 
-	rb_scan_args(argc, argv, "20:", &rb_target, &rb_reset_type, &rb_options);
-
 	Data_Get_Struct(self, git_repository, repo);
 
 	reset_type = parse_reset_type(rb_reset_type);
 	target = rugged_object_get(repo, rb_target, GIT_OBJ_ANY);
 
-	if (!NIL_P(rb_options)) {
-		VALUE rb_val;
-
-		rb_val = rb_hash_aref(rb_options, CSTR2SYM("signature"));
-		if (!NIL_P(rb_val))
-			signature = rugged_signature_get(rb_val, repo);
-
-		rb_val = rb_hash_aref(rb_options, CSTR2SYM("message"));
-		if (!NIL_P(rb_val))
-			log_message = StringValueCStr(rb_val);
-	}
-
-	error = git_reset(repo, target, reset_type, NULL, signature, log_message);
+	error = git_reset(repo, target, reset_type, NULL);
 
 	git_object_free(target);
-	git_signature_free(signature);
 
 	rugged_exception_check(error);
 
@@ -1866,10 +1906,10 @@ static void rugged_parse_checkout_options(git_checkout_options *opts, VALUE rb_o
 
 			if (rb_strategy == CSTR2SYM("safe")) {
 				opts->checkout_strategy |= GIT_CHECKOUT_SAFE;
-			} else if (rb_strategy == CSTR2SYM("safe_create")) {
-				opts->checkout_strategy |= GIT_CHECKOUT_SAFE_CREATE;
 			} else if (rb_strategy == CSTR2SYM("force")) {
 				opts->checkout_strategy |= GIT_CHECKOUT_FORCE;
+			} else if (rb_strategy == CSTR2SYM("recreate_missing")) {
+				opts->checkout_strategy |= GIT_CHECKOUT_RECREATE_MISSING;
 			} else if (rb_strategy == CSTR2SYM("allow_conflicts")) {
 				opts->checkout_strategy |= GIT_CHECKOUT_ALLOW_CONFLICTS;
 			} else if (rb_strategy == CSTR2SYM("remove_untracked")) {
@@ -1999,8 +2039,8 @@ static void rugged_parse_checkout_options(git_checkout_options *opts, VALUE rb_o
  *    :safe ::
  *      Allow safe updates that cannot overwrite uncommitted data.
  *
- *    :safe_create ::
- *      Allow safe updates plus creation of missing files.
+ *    :recreate_missing ::
+ *      Allow checkout to recreate missing files.
  *
  *    :force ::
  *      Allow all updates to force working directory to look like index.
@@ -2141,6 +2181,54 @@ static VALUE rb_git_checkout_tree(int argc, VALUE *argv, VALUE self)
 	return Qnil;
 }
 
+/**
+ *  call-seq: repo.checkout_index(index[,options]) -> nil
+ *
+ *  Updates files in the index and the working tree to match the content of the
+ *  commit pointed at by +index+.
+ *
+ *  See Repository#checkout_tree for a list of supported +options+.
+ */
+static VALUE rb_git_checkout_index(int argc, VALUE *argv, VALUE self)
+{
+	VALUE rb_index, rb_options;
+	git_repository *repo;
+	git_index *index;
+	git_checkout_options opts = GIT_CHECKOUT_OPTIONS_INIT;
+	struct rugged_cb_payload *payload;
+	int error, exception = 0;
+
+	rb_scan_args(argc, argv, "10:", &rb_index, &rb_options);
+
+	if (!rb_obj_is_kind_of(rb_index, rb_cRuggedIndex))
+		rb_raise(rb_eTypeError, "Expected Rugged::Index");
+
+	Data_Get_Struct(self, git_repository, repo);
+	Data_Get_Struct(rb_index, git_index, index);
+
+	rugged_parse_checkout_options(&opts, rb_options);
+
+	error = git_checkout_index(repo, index, &opts);
+	xfree(opts.paths.strings);
+
+	if ((payload = opts.notify_payload) != NULL) {
+		exception = payload->exception;
+		xfree(opts.notify_payload);
+	}
+
+	if ((payload = opts.progress_payload) != NULL) {
+		exception = payload->exception;
+		xfree(opts.progress_payload);
+	}
+
+	if (exception)
+		rb_jump_tag(exception);
+
+	rugged_exception_check(error);
+
+	return Qnil;
+}
+
 /**
  *  call-seq: repo.checkout_head([options]) -> nil
  *
@@ -2266,7 +2354,7 @@ static VALUE rb_git_repo_attributes(int argc, VALUE *argv, VALUE self)
 		VALUE rb_result;
 		const char **values;
 		const char **names;
-		int i, num_attr = RARRAY_LEN(rb_names);
+		long i, num_attr = RARRAY_LEN(rb_names);
 
 		if (num_attr > 32)
 			rb_raise(rb_eRuntimeError, "Too many attributes requested");
@@ -2412,6 +2500,9 @@ void Init_rugged_repo(void)
 	rb_define_method(rb_cRuggedRepo, "config",  rb_git_repo_get_config,  0);
 	rb_define_method(rb_cRuggedRepo, "config=",  rb_git_repo_set_config,  1);
 
+	rb_define_method(rb_cRuggedRepo, "ident", rb_git_repo_get_ident, 0);
+	rb_define_method(rb_cRuggedRepo, "ident=", rb_git_repo_set_ident, 1);
+
 	rb_define_method(rb_cRuggedRepo, "bare?",  rb_git_repo_is_bare,  0);
 	rb_define_method(rb_cRuggedRepo, "shallow?",  rb_git_repo_is_shallow,  0);
 	rb_define_method(rb_cRuggedRepo, "empty?",  rb_git_repo_is_empty,  0);
@@ -2429,7 +2520,7 @@ void Init_rugged_repo(void)
 
 	rb_define_method(rb_cRuggedRepo, "path_ignored?", rb_git_repo_is_path_ignored, 1);
 
-	rb_define_method(rb_cRuggedRepo, "reset", rb_git_repo_reset, -1);
+	rb_define_method(rb_cRuggedRepo, "reset", rb_git_repo_reset, 2);
 	rb_define_method(rb_cRuggedRepo, "reset_path", rb_git_repo_reset_path, -1);
 
 	rb_define_method(rb_cRuggedRepo, "namespace=", rb_git_repo_set_namespace, 1);
@@ -2440,6 +2531,7 @@ void Init_rugged_repo(void)
 	rb_define_method(rb_cRuggedRepo, "default_signature", rb_git_repo_default_signature, 0);
 
 	rb_define_method(rb_cRuggedRepo, "checkout_tree", rb_git_checkout_tree, -1);
+	rb_define_method(rb_cRuggedRepo, "checkout_index", rb_git_checkout_index, -1);
 	rb_define_method(rb_cRuggedRepo, "checkout_head", rb_git_checkout_head, -1);
 
 	rb_define_method(rb_cRuggedRepo, "cherrypick", rb_git_repo_cherrypick, -1);
